#include "core/LinkedList.h"

#include <stdio.h>
#include <string.h>

#include "core/ToString.h"
#include "core/Utility.h"

void coreDestroyLinkedList(CoreLinkedList* l) {
    coreClearLinkedList(l);
}

CoreLinkedListNode* coreLinkedListAddPointer(CoreLinkedList* l,
                                             const void* data, size_t n) {
    CoreLinkedListNode* node = coreAllocate(sizeof(CoreLinkedListNode) + n);
    node->next = nullptr;
    node->previous = nullptr;
    memcpy(coreLinkedListGetPointer(node, void), data, n);
    l->length++;
    if(l->first == nullptr) {
        l->first = node;
        l->last = node;
        return node;
    }
    l->last->next = node;
    node->previous = l->last;
    l->last = node;
    return node;
}

void coreClearLinkedList(CoreLinkedList* l) {
    CoreLinkedListNode* n = l->first;
    while(n != nullptr) {
        CoreLinkedListNode* next = n->next;
        coreFree(n);
        n = next;
    }
    l->length = 0;
    l->first = nullptr;
    l->last = nullptr;
}

void coreLinkedListRemove(CoreLinkedList* l, CoreLinkedListNode* n) {
    if(n == l->first) {
        if(l->first->next != nullptr) {
            l->first->next->previous = nullptr;
        }
        l->first = l->first->next;
    }
    if(n == l->last) {
        if(l->last->previous != nullptr) {
            l->last->previous->next = nullptr;
        }
        l->last = l->last->previous;
    }
    if(n->previous != nullptr) {
        n->previous->next = n->next;
    }
    if(n->next != nullptr) {
        n->next->previous = n->previous;
    }
    l->length--;
    coreFree(n);
}

void coreLinkedListRemoveFirst(CoreLinkedList* l) {
    coreLinkedListRemove(l, l->first);
}

void coreLinkedListRemoveLast(CoreLinkedList* l) {
    coreLinkedListRemove(l, l->last);
}

size_t coreToStringLinkedList(CoreLinkedList* l, char* buffer, size_t n,
                              CoreToString c) {
    size_t w = 0;
    coreStringAdd(&w, &buffer, &n, coreToString(buffer, n, "["));
    CoreLinkedListNode* current = l->first;
    while(current != nullptr) {
        coreStringAdd(&w, &buffer, &n,
                      c(coreLinkedListGetPointer(current, void), buffer, n));
        current = current->next;
        if(current != nullptr) {
            coreStringAdd(&w, &buffer, &n, coreToString(buffer, n, ", "));
        }
    }
    coreStringAdd(&w, &buffer, &n, coreToString(buffer, n, "]"));
    return w;
}

void coreSwapLinkedList(CoreLinkedList* a, CoreLinkedList* b) {
    CoreLinkedList tmp = *a;
    *a = *b;
    *b = tmp;
}
