#include "core/View.h"

#include "core/Generic.h"

#define CV3(a, b, c) (&(Vector3){{a, b, c}})

void initView(View* v) {
    *v = (View){0};
}

void updateDirections(View* v, float lengthAngle, float widthAngle) {
    angles(&v->front, lengthAngle, widthAngle);

    cross(&v->right, &v->front, &V(0.0f, 1.0f, 0.0f));
    normalize(&v->right);

    cross(&v->up, &v->right, &v->front);
    normalize(&v->up);

    invert(&v->left, &v->right);
    invert(&v->back, &v->front);
    invert(&v->down, &v->up);
}

void updateDirectionsQ(View* v, const Quaternion* q) {
    mul(&v->up, q, &V(0.0f, 1.0f, 0.0f));
    mul(&v->back, q, &V(-1.0f, 0.0f, 0.0f));

    cross(&v->right, &v->up, &v->back);
    normalize(&v->right);

    invert(&v->left, &v->right);
    invert(&v->front, &v->back);
    invert(&v->down, &v->up);
}

Matrix* updateMatrix(View* v, const Vector3* pos) {
    Vector4* d = v->view.data;
    d[0] = V(v->right.data[0], v->right.data[1], v->right.data[2],
             -dot(&v->right, pos));
    d[1] = V(v->up.data[0], v->up.data[1], v->up.data[2], -dot(&v->up, pos));
    d[2] = V(v->back.data[0], v->back.data[1], v->back.data[2],
             -dot(&v->back, pos));
    d[3] = V(0.0f, 0.0f, 0.0f, 1.0f);
    return &v->view;
}
